import os

import requests
from expiringdict import ExpiringDict
from rest_framework.utils import json
from django.conf import settings

try:
    from urllib.request import Request, urlopen  # Python 3
except ImportError:
    from urllib2 import Request, urlopen  # Python 2


class JSQLConnector:
    def checkIfObjectIsArray(self, object):
        return isinstance(object, list)

    def parseRequestToJSON(self, object):
        body_unicode = object.body.decode('utf-8')
        return json.loads(body_unicode)

    def parseStringToJSON(self, object):
        return json.loads(object)

    def retrieveDialect(self):
        headers = {'Content-Type': 'application/json', 'ApiKey': getattr(settings, "API_KEY", None),
                   'MemberKey': getattr(settings, "MEMBER_KEY", None)}
        resp = ""
        cache = ExpiringDict(max_len=100, max_age_seconds=10)
        cache["request"] = requests.get('http://localhost:9191/api/request/options/all', headers=headers)
        resp = cache.get("request").json()
        return {'code': resp["code"], 'data': resp}

    def getSQLQuery(self, token):
        headers = {'Content-Type': 'application/json', 'ApiKey': getattr(settings, "API_KEY", None),
                   'MemberKey': getattr(settings, "MEMBER_KEY", None)}
        resp = ""
        code = 100
        url = "http://localhost:9191/api/request/queries"
        if self.checkIfObjectIsArray(token):
            data = "[\""
            for t in token:
                data += t + "\", \""
            data = data[:-3] + "]"
            url += "/grouped"
        else:
            data = "[\"" + token + "\"]"
        r = requests.post(url, data=data, headers=headers)
        try:
            r.raise_for_status()
        except Exception:
            resp = json.loads(r.text)["message"]
        else:
            if self.checkIfObjectIsArray(r.json()):
                resp = r.json()[0]["query"]
                code = r.status_code
            else:
                resp = r.json()["description"]
                code = r.json()["code"]

        return {'code': code, 'data': resp}



from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.generic import UpdateView
from rest_framework import permissions
from rest_framework.response import Response
from rest_framework.utils import json
from rest_framework.views import APIView

from jsql.connector import JSQLConnector
from jsql.executor import JSQLExecutor


class SelectView(APIView):
    permission_classes = (permissions.AllowAny,)

    @classmethod
    def get_extra_actions(cls):
        return []

    def post(self, request, format=None):
        ex = JSQLExecutor()
        return Response(ex.executeSelect(request))


class InsertView(APIView):
    permission_classes = (permissions.AllowAny,)

    @classmethod
    def get_extra_actions(cls):
        return []

    def post(self, request, format=None):
        ex = JSQLExecutor()
        return Response(ex.executeInsert(request))


class UpdateView(APIView):
    permission_classes = (permissions.AllowAny,)

    @classmethod
    def get_extra_actions(cls):
        return []

    def post(self, request, format=None):
        ex = JSQLExecutor()
        return Response(ex.executeUpdateAndDelete(request))


class DeleteView(APIView):
    permission_classes = (permissions.AllowAny,)

    @classmethod
    def get_extra_actions(cls):
        return []

    def post(self, request, format=None):
        ex = JSQLExecutor()
        return Response(ex.executeUpdateAndDelete(request))



import re

from django.db import connection

from jsql.connector import JSQLConnector


class JSQLExecutor:
    cursor = connection.cursor()
    connector = JSQLConnector()
    paramsError = False

    def executeSelect(self, request):
        data = self.connector.parseRequestToJSON(request)
        query = self.connector.getSQLQuery(data["token"])
        params = None
        try:
            params = data["params"]
        except:
            pass
        if query["code"] >= 400:
            return {'code': query["code"], 'description': query["data"]}
        result = ""
        try:
            finalQuery = self.substituteParams(query["data"], params)
            if self.paramsError is True:
                self.paramsError = False
                return {'code': 400, 'description': finalQuery}
            self.cursor.execute(finalQuery)
            columns = self.cursor.description
            result = [{self.toCamelCase(columns[index][0]): column for index, column in enumerate(value)} for value in
                      self.cursor.fetchall()]
        except Exception as e:
            return {'code': 400, 'description': str(e)}

        return result

    def executeUpdateAndDelete(self, request):
        data = self.connector.parseRequestToJSON(request)
        query = self.connector.getSQLQuery(data["token"])
        params = None
        try:
            params = data["params"]
        except:
            pass
        if query["code"] >= 400:
            return {'code': query["code"], 'description': query["data"]}
        try:
            finalQuery = self.substituteParams(query["data"], params)
            if self.paramsError is True:
                self.paramsError = False
                return {'code': 400, 'description': finalQuery}
            self.cursor.execute(finalQuery)
        except Exception as e:
            return {'code': 400, 'description': str(e)}
        return {'status': 'OK'}

    def executeInsert(self, request):
        dialect = self.connector.retrieveDialect()
        if dialect["code"] != 200:
            return {'code': dialect["code"], 'description': dialect["data"]["description"]}
        data = self.connector.parseRequestToJSON(request)
        query = self.connector.getSQLQuery(data["token"])
        params = None
        try:
            params = data["params"]
        except:
            pass
        if query["code"] >= 400:
            return {'code': query["code"], 'description': query["data"]}
        result = ""
        finalQuery = self.substituteParams(query["data"], params)
        if self.paramsError is True:
            self.paramsError = False
            return {'code': 400, 'description': finalQuery}
        if dialect["data"]["data"]["databaseDialect"] == 'POSTGRES':
            try:
                self.cursor.execute(finalQuery + ' returning id')
                result = self.cursor.fetchall()
            except Exception as e:
                return {'code': 400, 'description': str(e)}
        if dialect["data"]["data"]["databaseDialect"] == 'MYSQL':
            try:
                self.cursor.execute(finalQuery)
                self.cursor.execute('SELECT LAST_INSERT_ID()')
                result = self.cursor.fetchall()
            except Exception as e:
                return {'code': 400, 'description': str(e)}
        return {"lastId": result}

    def toCamelCase(self, key):
        parts = key.split("_")
        camelCaseString = ""
        for i in range(len(parts)):
            if i != 0:
                parts[i] = parts[i][:1].upper() + parts[i][1:].lower()
            camelCaseString += parts[i]
        return camelCaseString

    def substituteParams(self, query, params):
        result = query
        splittedSQL = result
        if params is not None:
            if isinstance(params, dict):
                for key, value in params.items():
                    if isinstance(value, str):
                        result = result.replace(":" + key, "'" + str(value) + "'")
                    else:
                        result = result.replace(":" + key, str(value))
            else:
                for param in params:
                    if isinstance(param, str):
                        result = result.replace('?', "'" + str(param) + "'", 1)
                    else:
                        result = result.replace('?', str(param), 1)
        if isinstance(params, dict):
            splittedSQL = re.findall(r":\w+", result)
            errorMessage = "You have to include these params to your request: "
            for s in splittedSQL:
                if ":" in s and "'" not in s and "\"" not in s and "`" not in s:
                    errorMessage += s
                    self.paramsError = True
        else:
            if result.count("?") > 0:
                errorMessage = "You forgot to pass " + str(result.count("?")) + " parameters"
                self.paramsError = True
        if self.paramsError is True:
            return errorMessage
        return result



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="jsql-django",
    version="1.0.1",
    author="JSQL Sp. z o.o.",
    author_email="office@jsql.it",
    description="JSQL backend plugin for Django app",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.jsql.it",
    packages=setuptools.find_packages(),
    install_requires=["Django", "djangorestframework", "Pillow", "psycopg2", "pytz", "requests", "expiringdict"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
